use crate::utils::{is_prime, mod_pow};
use gcd::Gcd;
/**
 * Calculate the prime divisor for some number `n`
 *
 * Uses cycle finding
*/
pub fn pollard_rho(n: u32) -> u32 {
    if n == 1 {
        return 1;
    } // 1 only has 1 as prime divisor
    if n % 2 == 0 {
        return 2;
    } // even numbers have at least 2 as prime divisor
    if is_prime(n) {
        return n;
    } // need to detect to be able to decide if we have bad random numbers or the

    let mut x = rand::random::<u32>() % n; // cycle finding: tortoise
    let mut y = x; // cycle finding: hare

    let c = rand::random::<u32>() % n; // random number to add to the cycle finding moves
    let mut div = 1; // divisor - 1 applies to all numbers and is our failed and start value

    while div == 1 {
        // tortoise move
        x = (mod_pow(x, 2, n) + c + n) % n;
        // hare move
        y = (mod_pow(y, 2, n) + c + n) % n;
        y = (mod_pow(y, 2, n) + c + n) % n;
        // divisor is the greatest common divisor between |x-y| and n
        div = u32::try_from((i32::try_from(x).unwrap() - i32::try_from(y).unwrap()).abs())
            .unwrap()
            .gcd(n);
        // bad random numbers, try again
        if div == n {
            return pollard_rho(n);
        }
    }

    div
}
